<?php
/*
Plugin Name: Analytify
Plugin URI: http://wp-analytify.com/
Description: Analytify makes Google Analytics simple for everything in WordPress (posts,pages etc). It presents the statistics in a beautiful way under the WordPress Posts/Pages at front end, backend and in its own Dashboard. This provides Stats from Country, Referrers, Social media, General stats, New visitors, Returning visitors, Exit pages, Browser wise and Top keywords. This plugin provides the RealTime statistics in a new UI that is easy to understand and looks good.
Version: 1.0.5
Author: Adnan
Author URI: http://adnan.pk/
Text Domain: wp-analytify
*/

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;

ini_set( 'include_path', dirname(__FILE__) . '/lib/' );

if (! class_exists( 'WP_Analytify' ) ) {

    // Setting Global Values
    define( 'ANALYTIFY_LIB_PATH', dirname(__FILE__) . '/lib/' );
    define( 'ANALYTIFY_ID', 'wp-analytify-options' );
    define( 'ANALYTIFY_NICK', 'Analytify' );
    define( 'ANALYTIFY_ROOT_PATH', dirname(__FILE__) );
    define( 'ANALYTIFY_VERSION', '1.0.5');
    //Grab ClientID and ClientSecret from https://console.developers.google.com/ after creating a project there.
    define( 'ANALYTIFY_CLIENTID', '707435375568-ujudvn46afvmh093lacrqrakui22dk6d.apps.googleusercontent.com' );
    define( 'ANALYTIFY_CLIENTSECRET', 'bVsFyV0eBsWSKc_OfKGcyS_9' ); 
    define( 'ANALYTIFY_REDIRECT', 'urn:ietf:wg:oauth:2.0:oob' );  // This will redirect to window where we can copy Access code.
    define( 'ANALYTIFY_SCOPE', 'https://www.googleapis.com/auth/analytics' ); // readonly scope
    define( 'ANALYTIFY_DEV_KEY', 'AIzaSyDXjBezSlaVMPk8OEi8Vw5aFvteouXHZpI');

    set_site_transient( 'update_plugins', null );

    define( 'ANALYTIFY_STORE_URL', 'http://wp-analytify.com' );
    define( 'ANALYTIFY_PRODUCT_NAME', 'Analytify WordPress Plugin' );

class WP_Analytify {

    public $token  = false;
    public $client = null;

    // Constructor
    function __construct() {

        if (! class_exists('Analytify_Google_Client') ) {

            require_once ANALYTIFY_LIB_PATH . 'Google/Client.php';
            require_once ANALYTIFY_LIB_PATH . 'Google/Service/Analytics.php';

        }

        if( !class_exists( 'EDD_SL_Plugin_Updater' ) ) {
            // load plugin updater
            include( dirname( __FILE__ ) . '/lib/EDD_SL_Plugin_Updater.php' );
        }

        $this->client = new Analytify_Google_Client();
        $this->client->setApprovalPrompt( "force" );
        $this->client->setAccessType( 'offline' );
        $this->client->setClientId( ANALYTIFY_CLIENTID );
        $this->client->setClientSecret( ANALYTIFY_CLIENTSECRET );
        $this->client->setRedirectUri( ANALYTIFY_REDIRECT );
        $this->client->setScopes( ANALYTIFY_SCOPE ); 
        $this->client->setDeveloperKey( ANALYTIFY_DEV_KEY);

        try{
            $this->service = new Analytify_Google_Service_Analytics( $this->client );
        }
        catch ( Google_ServiceException $e ) {
            
        }
        $this->pa_connect();
        add_action( 'plugin_action_links', array( 
                    $this,
                    'pa_plugin_links'
                ),10,2);

        add_action( 'admin_enqueue_scripts', array( 
                    $this,
                    'pa_scripts'
                ));
        add_action( 'admin_enqueue_scripts', array( 
                    $this,
                    'pa_styles'
                ));
        add_action( 'admin_menu', array(
                    $this,
                    'pa_add_menu'
                ));

        add_action( 'admin_init', array(
                    $this,
                    'analytify_plugin_updater'
                ));

        add_action('admin_init', array($this, 'wp_analytify_register_option' ) );
        add_action('admin_init', array($this, 'analytify_activate_license' ) );
        add_action('admin_init', array($this, 'analytify_deactivate_license' ) );


		if( get_option( 'analytify_code') == 1) {

		    add_action( 'wp_head', array(
		                    $this,
		                    'Analytify_add_analytics_code'
		                ));
		}

        add_action( 'wp_ajax_nopriv_get_ajax_single_admin_analytics', array(
                    $this,
                    'get_ajax_single_admin_analytics'
                ));

        add_action( 'wp_ajax_get_ajax_single_admin_analytics', array(
                    $this,
                    'get_ajax_single_admin_analytics'
                ));

        if( get_option( 'analytify_disable_front') == 0 ){
            add_filter( 'the_content', array(
                        $this,
                        'display_stats'
                ));
        }

        add_action ( 'wp_ajax_pa_get_online_data', array (
                    $this,
                    'pa_realtime_data_get'
                ));
        /* 
         * load Analytics under the EDIT POST Screen
         * add action runs only for admin section and load metabox.
         */

        if ( is_admin() ) {
                    add_action( 'load-post.php', array(
                    $this,
                    'load_metaboxes'
                ));
        }

        // Show welcome message when user activate plugin.
        if ( get_option( 'pa_welcome_message' ) == 0 ) {
            add_action( 'admin_print_footer_scripts', array( $this, 'pa_welcome_message' ) );
        }      

        // localization language file filter
        add_action( 'init', array($this, 'analytify_textdomain') );
        //add_action( 'admin_footer', array( &$this, 'profile_warning' ) );
    }

    /*
     * Set plugin language/localizations files directory
     */
    public function analytify_textdomain() {

        $plugin_dir = basename( dirname(__FILE__) );
        load_plugin_textdomain( 'wp-analytify', false , $plugin_dir . '/lang/');

    }

    /*
     * Show analytics sections under the posts/pages in the metabox.
     */

    function load_metaboxes() {
        add_action( 'add_meta_boxes', array(
                    $this,
                    'show_admin_single_analytics_add_metabox'
        ));
    }

    /* 
     * Show metabox under each Post type to display Analytics of single post/page in wp-admin
     */
    public function show_admin_single_analytics_add_metabox() {

            $post_types = get_post_types();

            foreach ( $post_types as $post_type ) {
                add_meta_box('pa-single-admin-analytics', // $id
                    'Google Analytics of above page.', // $title
                    array(
                        'WP_Analytify',
                        'show_admin_single_analytics'
                    ), // $callback
                    $post_type, // $posts
                    'normal',   // $context
                    'high'      // $priority
                ); 
            } //$post_types as $post_type
    }

    /* 
     * Show Analytics of single post/page in wp-admin
     */
    public static function show_admin_single_analytics() {

            global $post;
            $urlPost = '';
            $wp_analytify  = new WP_Analytify();
            $urlPost = parse_url( get_permalink( $post->ID ) );
            $start_date = ''; $end_date = ''; $urlpost = '' ;
            $is_access_level = get_option("post_analytics_access_back");
            if( $wp_analytify->pa_check_roles( $is_access_level ) ){ ?>

                <div class="pa-filter">
                    <table cellspacing="0" cellpadding="0" width="400">
                        <tbody>
                            <tr>
                                <td width="0">
                                    <input type="text" id="start_date" name="start_date">
                                </td>
                                <td width="0">
                                    <input type="text" id="end_date" name="end_date">
                                </td>
                                    <input type="hidden" name="urlpost" id="urlpost" value="<?php echo $urlPost['path']; ?>">
                                <td width="0">
                                    <input type="button" id="view_analytics" name="view_analytics" value="View Analytics" class="button-primary btn-green">
                                </td>
                           </tr>
                        </tbody>
                    </table>
                </div>
                <div class="loading" style="display:none">
                    <img src="<?php echo plugins_url('images/loading.gif', __FILE__);?>">
                </div>
                <?php
                $wp_analytify->get_single_admin_analytics( $start_date, $end_date, $post->ID );
            }
            else{
                    echo _e( 'You are not allowed to see stats', 'wp-analytify' );
            }
    }

    // Add Google Analytics JS code
    public function Analytify_add_analytics_code() {
    ?>
    
       <!-- This code is added by Analytify v - <?php echo ANALYTIFY_VERSION; ?> !-->
        <?php
        if( get_option( 'analytify_tracking_code' ) == 'universal' ) { ?>

            <script>
                  (function(i,s,o,g,r,a,m){i['GoogleAnalyticsObject']=r;i[r]=i[r]||function(){
                  (i[r].q=i[r].q||[]).push(arguments)},i[r].l=1*new Date();a=s.createElement(o),
                  m=s.getElementsByTagName(o)[0];a.async=1;a.src=g;m.parentNode.insertBefore(a,m)
                  })(window,document,'script','//www.google-analytics.com/analytics.js','ga');
                  ga('create', '<?php echo get_option( "webPropertyId" );?>', 'auto');
                  ga('send', 'pageview');
            </script>

        <?php 
        }

        if( get_option( 'analytify_tracking_code' ) == 'ga' ) { ?>
            
            <script type="text/javascript">//<![CDATA[
                var _gaq = _gaq || [];
                _gaq.push(['_setAccount', '<?php echo get_option( "webPropertyId" );?>']);
                _gaq.push(['_trackPageview']);
                (function () {
                    var ga = document.createElement('script');
                    ga.type = 'text/javascript';
                    ga.async = true;
                    ga.src = ('https:' == document.location.protocol ? 'https://ssl' : 'http://www') + '.google-analytics.com/ga.js';
                    var s = document.getElementsByTagName('script')[0];
                    s.parentNode.insertBefore(ga, s);
                })();
                //]]>
            </script>

        <?php
        }
        ?>
      <!-- This code is added by Analytify  v - <?php echo ANALYTIFY_VERSION; ?>!-->
    <?php
    }

    // Get RealTime statistics.
    function pa_realtime_data_get() { 
            
        if (! isset( $_REQUEST['pa_security'] ) OR ! wp_verify_nonce( $_REQUEST['pa_security'] , 'pa_get_online_data' ) ) {
            return;
        }
            
        if (! function_exists( 'curl_version' ) ) {
            die();
        }

        print_r( stripslashes( json_encode( $this->pa_realtime_data( get_option( "pt_webprofile_dashboard" ) ) ) ) ); 

        die();
    }

    /**
    * Add a link to the settings page to the plugins list
    */
    public function pa_plugin_links( $links, $file ) {

            static $this_plugin;
            if ( empty( $this_plugin ) ) 
                $this_plugin = 'wp-analytify/wp-analytify.php';
            if ( $file == $this_plugin ) {
                $settings_link = '<a href="' . admin_url("admin.php?page=analytify-settings") . '">' . __( 'Settings', 'wp-analytify' ) . '</a> | <a href="' . admin_url("admin.php?page=analytify-dashboard") . '">' . __( 'Dashboard', 'wp-analytify' ) . '</a>';
                array_unshift( $links, $settings_link );
            }
            return $links;

    }

    /**
     * Display warning if profiles are not selected.
     */
    public function pa_check_warnings(){
            add_action( 'admin_footer', array( &$this, 'profile_warning' ) );
    }

    /**
     * Get current screen details
     */
    public static function pa_page_file_path() {
    
            $screen = get_current_screen();

            if( strpos( $screen->base, 'analytify-settings' ) !== false ) {
                include( ANALYTIFY_ROOT_PATH . '/inc/options-settings.php' );
            } 
            elseif( strpos($screen->base, 'analytify-extensions') !== false ) {
                include( ANALYTIFY_ROOT_PATH . '/inc/analytics-extensions.php' );
            } 
            else {
                include( ANALYTIFY_ROOT_PATH . '/inc/analytics-dashboard.php' );
            }
    }
    /**
     * Styling: loading stylesheets for the plugin.
     */
    public function pa_styles( $page ) {
            wp_enqueue_style( 'wp-analytify-style', plugins_url('css/wp-analytify-style.css', __FILE__));
            wp_enqueue_style( 'ui-css', plugins_url('jquery-ui-theme/jquery-ui.css', __FILE__));
            wp_enqueue_style( 'select-bootstrap-css', plugins_url('css/select2-bootstrap.css', __FILE__));
            wp_enqueue_style( 'select-css', plugins_url('css/select2.css', __FILE__));
            wp_enqueue_style( 'jquery-ui-tooltip-css', plugins_url('css/jquery.ui.tooltip.html.css', __FILE__) );

            if ( get_option( 'pa_welcome_message' ) == '0' ) {
                wp_enqueue_style( 'wp-pointer' );
            }
    }

    /**
     * Loading scripts js for the plugin.
     */
    public function pa_scripts( $page ) {
            // $pa_ext =   new WP_Pa_Extensions();
            wp_enqueue_script ('jquery');
            wp_enqueue_script ('charts_api_js', 'https://www.google.com/jsapi', false, '1.0.0');
            wp_enqueue_script ('select-js', plugins_url('js/select2.js', __FILE__), false, '1.0.0');
            wp_enqueue_script ('script-js', plugins_url('js/wp-analytify.js', __FILE__), false, '1.0.0');
            wp_enqueue_script ( "jquery-ui-tooltip" );
            wp_enqueue_script ('jquery-ui-datepicker');
            if ( get_option('pa_welcome_message') == '0' ) wp_enqueue_script( 'wp-pointer' );
            //wp_localize_script( 'script-js', 'pa_extensions', array( 'extensions' => $pa_ext->prepare_extensions_for_js( "my" ) ) );

    }

    /** 
     * Create Analytics menu at the left side of dashboard
     */
    public static function pa_add_menu() {
            
            add_menu_page( ANALYTIFY_NICK, 'Analytify', 'manage_options', 'analytify-dashboard', array(
                     __CLASS__,
                    'pa_page_file_path'
                ), plugins_url('images/wp-analytics-logo.png', __FILE__));

            add_submenu_page( 'analytify-dashboard', ANALYTIFY_NICK . ' Dashboard', ' Dashboard', 'manage_options', 'analytify-dashboard', array(
                    __CLASS__,
                    'pa_page_file_path'
                ));
            
            add_submenu_page( 'analytify-dashboard', ANALYTIFY_NICK . ' Settings', '<b style="color:#f9845b">Settings</b>', 'manage_options', 'analytify-settings', array(
                    __CLASS__,
                    'pa_page_file_path'
                ));
    }

    /** 
     * 
     */
    public function pa_settings_tabs( $current = 'authentication' ) {
            
            $tabs = array( 'authentication' =>  'Authentication', 
                            //'general'       =>  'General', 
                            'profile'       =>  'Profile',
                            'front'         =>  'Front' ,
                            'admin'         =>  'Admin',  
                            'license'        =>  'License'  
                    );

            echo '<div id="icon-themes" class="icon32"></div>';
            echo '<div class="left-area">';
            echo '<h2 class="nav-tab-wrapper">';
            foreach( $tabs as $tab => $name ){

                $class = ( $tab == $current ) ? ' nav-tab-active' : '';
                echo "<a class='nav-tab$class' href='?page=analytify-settings&tab=$tab'>$name</a>";
            }
            echo '</h2>';
    }

    /**
     * Connect with Google Analytics API and get authentication token and save it.
     */
    public function pa_connect() {
            $ga_google_authtoken = get_option('pa_google_token');

            if (! empty( $ga_google_authtoken )) {
                $this->client->setAccessToken( $ga_google_authtoken );
            } //!empty($ga_google_authtoken)
            else{
                $authCode = get_option( 'post_analytics_token' );
                
                if (empty($authCode)) return false;

                try {
                    $accessToken = $this->client->authenticate( $authCode );
                }
                catch (Exception $e) {
                    return false;
                }

                if ($accessToken) {
                    $this->client->setAccessToken( $accessToken );
                    update_option( 'pa_google_token', $accessToken );
                    return true;
                } //$accessToken
                else {
                    return false;
                }
            }
            $this->token = $this->client->getAccessToken();
            return true;
    }

    /**
     * Get profiles from user Google Analytics account profiles.
     */
    public function pt_get_analytics_accounts() {
            try {
                if( get_option('pa_google_token') !='' ) {
                    $profiles = $this->service->management_profiles->listManagementProfiles("~all", "~all");
                    return $profiles;
                }
                else{
                    echo _e( 'Must be login to access profiles', 'wp-analytify' );
                }
            }
            catch (Exception $e) {
                die('An error occured: ' . $e->getMessage() . '\n');
            }
    }

    public function pa_setting_url() {
        return admin_url('admin.php?page=pa-settings');
    }


    public function pt_save_data( $key_google_token ) {
        update_option( "post_analytics_token", $key_google_token );
        $this->pa_connect();
        return true;
    }

    /**
     * Warning messages.
     */
    public function profile_warning() {

            $profile_id     =   get_option( "pt_webprofile" );
            $acces_token    =   get_option( "post_analytics_token" );

            if(! isset( $acces_token ) || empty( $acces_token )) {
               echo "<div id='message' class='error'><p><strong>" . __( "Analytify is not active. Please <a href='" . menu_page_url ( 'analytify-settings', false ) ."'>Authenticate</a> in order to get started using this plugin.", 'wp-analytify' )."</p></div>"; 
            }
            else{
                if(! isset( $profile_id ) || empty( $profile_id )){
                    echo '<div class="error"><p><strong>' . __( 'Google Analytics Profile is not set. Set the <a href="' . menu_page_url ( 'analytify-settings', false ) . '&tab=profile">Profile</a> ', 'wp-analytify' ) . '</p></div>';
                }
            }
    }


    // Showing stats on front end posts/pages
    public function display_stats( $content ) {
        global $post, $wp_analytify;

        if ( is_single() || is_page()) {

            // Showing stats to guests
            if( get_option( "post_analytics_showguest") == 1 || $wp_analytify->pa_check_roles( get_option( 'post_analytics_access' ) )) {

                $urlPost =  parse_url( get_permalink( $post->ID ) );
            
                if($urlPost['host'] == 'localhost') 
                    $filter = 'ga:pagePath==/'; //.$u_post['path'];
                else 
                    $filter = 'ga:pagePath==' .$urlPost['path']. '';        


                $stats   =  $this->pa_get_analytics( 'ga:sessions,ga:bounces,ga:entrances,ga:pageviews,ga:sessionDuration,ga:newVisits', get_the_time('Y-m-d', $post->ID), date('Y-m-d'), false, false, $filter );
                
                if ( isset( $stats->totalsForAllResults )) {
                    
                    if ( $stats->totalsForAllResults['ga:entrances'] <= 0 ) {
                        $entrances = '<td><strong>0.00%</strong></td>';
                    } //$stats->totalsForAllResults['ga:entrances'] <= 0
                    else{
                        $entrances = "<td><strong>" . number_format( round(( $stats->totalsForAllResults['ga:bounces'] / $stats->totalsForAllResults['ga:entrances'] ) * 100, 2), 2 ) . "%</strong></td>";
                    }
                    
                    if ( $stats->totalsForAllResults['ga:sessions'] <= 0 ) {
                        $timeOnSite = '<td><strong>00:00:00</strong></td>';
                    } //$stats->totalsForAllResults['ga:sessions'] <= 0
                    else {
                        $timeOnSite = "<td><strong>" . $this->pa_pretty_time( $stats->totalsForAllResults['ga:sessionDuration'] / $stats->totalsForAllResults['ga:sessions'] ) . "</strong></td>";
                    }

                    if ( $stats->totalsForAllResults['ga:sessions'] <= 0 ) {
                        $newVisits = '<td><strong>0.00%</strong></td>';
                    } //$stats->totalsForAllResults['ga:sessions'] <= 0
                    else {
                        $newVisits = "<td><strong>" . number_format( round(( $stats->totalsForAllResults['ga:newVisits'] / $stats->totalsForAllResults['ga:sessions'] ) * 100, 2), 2) . "%</strong></td>";
                    }

                    if ( $stats->totalsForAllResults['ga:sessions'] <= 0 ) {
                        $pageviews = '<td><strong>0.00</strong></td>';
                    } //$stats->totalsForAllResults['ga:sessions'] <= 0
                    else {
                        $pageviews = "<td><strong>" . number_format( round($stats->totalsForAllResults['ga:pageviews'] / $stats->totalsForAllResults['ga:sessions'], 2), 2) . "</strong></td>";
                    }

                    $content .= '
                        <table width="100%">
                            <tr>
                            <td width=""><strong>' . number_format($stats->totalsForAllResults['ga:sessions']) . '</strong></td>
                            <td width="">Visits</td>' . $entrances . '<td width="">Bounce Rate</td>
                            ' . $pageviews . '<td width="">page views/Visits</td>
                            </tr> 
                            <tr>
                            <td width=""><strong>' . number_format($stats->totalsForAllResults['ga:pageviews']) . '</strong></td>
                            <td width="">Pageviews</td>' . $timeOnSite . '<td width="">Avg. Time on Site</td>
                            ' . $newVisits . '<td>New Visits</td>
                            </tr> 
                        </table>';
                    } 
                }
            

            }
            return $content;
    }


    /*
     * get the Analytics data from ajax() call
     * 
     */

    public function get_ajax_single_admin_analytics() {

        $startDate = '';
        $endDate   = '';
        $postID    = 0 ;

        $startDate = $_POST['start_date'];
        $endDate   = $_POST['end_date'];
        $postID   = $_POST['post_id'];

        $this->get_single_admin_analytics( $startDate, $endDate, $postID );
        die();
    }


    public function get_single_admin_analytics( $start_date = '', $end_date = '', $postID = 0 ) {

            if( $this->analytify_check_license() == 'invalid') {

                echo _e( 'You must have proper license to see the Statistics. <br /> Activate it <a href="' . menu_page_url ( 'analytify-settings', false ) .'&tab=license">here</a> or Please buy it from <a href="http://wp-analytify.com">Our Site.</a>', 'wp-analytify' );
                return false;
            }

            global $post;
            
            //$urlPost = parse_url( get_permalink( $post->ID ) );
            
            if ( $postID == 0 ) {
                    $u_post = '/'; //$urlPost['path'];
            }
            else{
                    $u_post = parse_url( get_permalink( $postID ) );
            }

            if($u_post['host'] == 'localhost') 
                $filter = 'ga:pagePath==/youtube-blocked-in-pakistan/'; //.$u_post['path'];
            else 
                $filter = 'ga:pagePath==' .$u_post['path']. '';

            if ($start_date == '') {
                    $s_date = get_the_time('Y-m-d', $post->ID);
                    if(get_the_time('Y', $post->ID) < 2005){
                        $s_date = '2005-01-01';
                    }
            }
            else{
                    $s_date = $start_date;
            }

            if ($end_date == '') {
                    $e_date = date('Y-m-d');
            } //$start_date == ''
            else{
                $e_date = $end_date;
            }
            //echo $e_date;
            $show_settings = array();
            $show_settings = get_option('post_analytics_settings_back');
            ?>

            <div class="show-hide">
                <?php
                //print_r($u_post);
                
                echo '<p> Displaying Analytics of this page from ' . date("jS F, Y", strtotime($s_date)) . ' to '. date("jS F, Y", strtotime($e_date)) . '</p>';
                // Real time stats //
                
               // include ANALYTIFY_ROOT_PATH . '/views/admin/realtime-stats.php'; 
               // pa_include_realtime($this);

                // End Real time stats //
            if(! empty( $show_settings )){

                if (is_array( $show_settings )){

                        if (in_array( 'show-overall-back', $show_settings ))
                                { 
                                    $stats = $this->pa_get_analytics( 'ga:sessions,ga:bounces,ga:newUsers,ga:entrances,ga:pageviews,ga:sessionDuration,ga:avgSessionDuration,ga:users',$s_date, $e_date, false, false, $filter);
                                    if ( isset( $stats->totalsForAllResults ) ) 
                                        {
                                            include ANALYTIFY_ROOT_PATH . '/views/admin/general-stats.php'; 
                                            pa_include_general( $this, $stats);
                                        } 
                                }
                    }
            
            if (is_array( $show_settings )){
                if (in_array( 'show-country-back', $show_settings )){
                    $country_stats = $this->pa_get_analytics( 'ga:sessions', $s_date, $e_date, 'ga:country', '-ga:sessions', $filter,5);
                    if ( isset( $country_stats->totalsForAllResults ) ) { 
                        include ANALYTIFY_ROOT_PATH . '/views/admin/country-stats.php'; 
                        pa_include_country( $this, $country_stats );
                    }
                }
            }

            if (is_array( $show_settings )){
                if (in_array( 'show-city-back', $show_settings )){
                    $city_stats = $this->pa_get_analytics( 'ga:sessions', $s_date, $e_date, 'ga:city', '-ga:sessions', $filter, 5);
                    if ( isset( $city_stats->totalsForAllResults ) ) { 
                        include ANALYTIFY_ROOT_PATH . '/views/admin/city-stats.php'; 
                        pa_include_city( $this, $city_stats );
                    }
                }
            }
           

            if (is_array( $show_settings )){
                    if (in_array( 'show-keywords-back', $show_settings)){      
                    $keyword_stats = $this->pa_get_analytics( 'ga:sessions', $s_date, $e_date, 'ga:keyword', '-ga:sessions',$filter,10 );
                        if ( isset( $keyword_stats->totalsForAllResults ) ) {
                            include ANALYTIFY_ROOT_PATH . '/views/admin/keywords-stats.php'; 
                            pa_include_keywords( $this, $keyword_stats );
                        }
                    }
            }
            if (is_array($show_settings)){
                if (in_array( 'show-social-back', $show_settings )){       
                    $social_stats = $this->pa_get_analytics( 'ga:sessions', $s_date, $e_date, 'ga:socialNetwork', '-ga:sessions',$filter, 10);
                        if ( isset( $social_stats->totalsForAllResults ) ) { 
                            include ANALYTIFY_ROOT_PATH . '/views/admin/social-stats.php';
                            pa_include_social( $this, $social_stats );
                        } 
                }
            }
            if (is_array($show_settings)){
                if (in_array( 'show-browser-back', $show_settings )){     
                    $browser_stats = $this->pa_get_analytics( 'ga:sessions', $s_date, $e_date, 'ga:browser,ga:operatingSystem', '-ga:sessions',$filter,5 );
                        if (isset( $browser_stats->totalsForAllResults ) ) {
                            include ANALYTIFY_ROOT_PATH . '/views/admin/browser-stats.php'; 
                            pa_include_browser( $this, $browser_stats );
                        
                }
            }
            }

            if (is_array($show_settings)){
                if (in_array( 'show-os-back', $show_settings )){
                    $os_stats = $this->pa_get_analytics_dashboard( 'ga:sessions', $s_date, $e_date, 'ga:operatingSystem,ga:operatingSystemVersion', '-ga:sessions', $filter, 5);
                    if (isset( $os_stats->totalsForAllResults ) ) {
                        include ANALYTIFY_ROOT_PATH . '/views/admin/os-stats.php'; 
                        pa_include_operating( $this, $os_stats );
                        
                    }
                }
            }

            if (is_array($show_settings)){
                    if (in_array( 'show-mobile-back', $show_settings )){
                    $mobile_stats = $this->pa_get_analytics_dashboard( 'ga:sessions', $s_date, $e_date, 'ga:mobileDeviceInfo', '-ga:sessions', $filter, 5);     
                            if (isset( $mobile_stats->totalsForAllResults ) ) {
                                include ANALYTIFY_ROOT_PATH . '/views/admin/mobile-stats.php'; 
                                pa_include_mobile( $this, $mobile_stats );
                            
                    }
                }
            }

            if (is_array($show_settings)){
                if (in_array( 'show-referrer-back', $show_settings )){    
                    $referr_stats = $this->pa_get_analytics( 'ga:sessions', $s_date, $e_date, 'ga:source,ga:medium', '-ga:sessions',$filter,10 );
                    if ( isset( $referr_stats->totalsForAllResults ) ) {
                        include ANALYTIFY_ROOT_PATH . '/views/admin/referrers-stats.php'; 
                        pa_include_referrers( $this, $referr_stats );
                    }
                }
            }
            if (is_array( $show_settings )){
                if (in_array( 'show-pages-back', $show_settings )){    
                    $page_stats = $this->pa_get_analytics( 'ga:entrances,ga:pageviews,ga:exits', $s_date, $e_date, 'ga:PagePath', '-ga:exits', $filter, 5 );
                    if ( isset( $page_stats->totalsForAllResults ) ) {
                        include ANALYTIFY_ROOT_PATH . '/views/admin/single-pages-stats.php'; 
                        pa_single_pages_stats( $this, $page_stats );
                    }
                }
            }
        }
            else
                {
                    $stats = $this->pa_get_analytics( 'ga:sessions,ga:bounces,ga:newUsers,ga:entrances,ga:pageviews,ga:sessionDuration,ga:avgSessionDuration,ga:users',$s_date, $e_date, false, false, $filter );
                        if ( isset( $stats->totalsForAllResults ) ) 
                            {
                                include ANALYTIFY_ROOT_PATH . '/views/admin/general-stats.php'; 
                                pa_include_general( $this, $stats );
                            } 
                }
        ?>
            </div>   
    <?php
    }

    // This function grabs the data from Google Analytics
    public function pa_get_analytics( $metrics, $startDate, $endDate, $dimensions = false, $sort = false, $filter = false, $limit = false ){

            $this->service = new Analytify_Google_Service_Analytics($this->client);
            $params        = array();
            if ($dimensions){
                $params['dimensions'] = $dimensions;
            } //$dimensions
            if ($sort){
                $params['sort'] = $sort;
            } //$sort
            if ($filter){
                $params['filters'] = $filter;
            } //$filter
            if ($limit){
                $params['max-results'] = $limit;
            } //$limit
            $profile_id = get_option("pt_webprofile");

            if (!$profile_id){
                return false;
            }
            
            return $this->service->data_ga->get('ga:' . $profile_id, $startDate, $endDate, $metrics, $params);
    }


    public function pa_get_analytics_dashboard($metrics, $startDate, $endDate, $dimensions = false, $sort = false, $filter = false, $limit = false){

            $this->service = new Analytify_Google_Service_Analytics($this->client);
            $params        = array();
            if ($dimensions){
                $params['dimensions'] = $dimensions;
            }
            if ($sort){
                $params['sort'] = $sort;
            } 
            if ($filter){
                $params['filters'] = $filter;
            } 
            if ($limit){
                $params['max-results'] = $limit;
            } 
            
            $profile_id = get_option("pt_webprofile_dashboard");
            if (!$profile_id){
                return false;
            }
            
            return $this->service->data_ga->get('ga:' . $profile_id, $startDate, $endDate, $metrics, $params);
    }

    function pa_pretty_time( $time ) {
            // Check if numeric
            if (is_numeric($time))
                {
                $value = array(
                    "years" => '00',
                    "days" => '00',
                    "hours" => '',
                    "minutes" => '',
                    "seconds" => ''
                );
                if ($time >= 31556926)
                    {
                    $value["years"] = floor($time / 31556926);
                    $time           = ($time % 31556926);
                    } //$time >= 31556926
                if ($time >= 86400)
                    {
                    $value["days"] = floor($time / 86400);
                    $time          = ($time % 86400);
                    } //$time >= 86400
                if ($time >= 3600)
                    {
                    $value["hours"] = str_pad(floor($time / 3600), 1, 0, STR_PAD_LEFT);
                    $time           = ($time % 3600);
                    } //$time >= 3600
                if ($time >= 60)
                    {
                    $value["minutes"] = str_pad(floor($time / 60), 1, 0, STR_PAD_LEFT);
                    $time             = ($time % 60);
                    } //$time >= 60
                    $value["seconds"] = str_pad(floor($time), 1, 0, STR_PAD_LEFT);
                # Get the hour:minute:second version
                if($value['hours']!=''){
                    $attach_hours='<sub>h</sub> ';
                }
                if($value['minutes']!=''){
                    $attach_min='<sub>min</sub> ';
                }
                if($value['seconds']!=''){
                    $attach_sec='<sub>sec</sub>';
                }
                return $value['hours'] .@$attach_hours. $value['minutes'] .@$attach_min. $value['seconds'].$attach_sec;
                //return $value['hours'] . ':' . $value['minutes'] . ':' . $value['seconds'];
                } //is_numeric($time)
            else
                {
                return false;
                }
    }

    public function pa_check_roles($access_level) {
            if (is_user_logged_in () && isset ( $access_level )) {
                global $current_user;
                $roles = $current_user->roles;

                if ((current_user_can ( 'manage_options' ))) {
                    return true;
                }
                if (in_array ( $roles[0], $access_level )) {
                    return true;
                } else {
                    return false;
                }
            }
    }

    public function pa_realtime() {
            $code = '
                <script type="text/javascript">
                function onlyUniqueValues(value, index, self) {
                    return self.indexOf(value) === index;
                 }
        
                function countvisits(data, searchvalue) {
                    var count = 0;
                    if(data["rows"]){
                    for ( var i = 0; i < data["rows"].length; i = i + 1 ) {
                        if (jQuery.inArray(searchvalue, data["rows"][ i ])>-1){
                            count += parseInt(data["rows"][ i ][6]);
                        }
                    }
                }
                    return count;
                 }
        
                function pa_generatetooltip(data) {
                    var count = 0;
                    var table = "";
                    for ( var i = 0; i < data.length; i = i + 1 ) {
                            count += parseInt(data[ i ].count);
                            table += "<td>"+data[i].value+"</td><td class=\'pa-pgdetailsr\'>"+data[ i ].count+"</td></tr>";
                    };
                    if (count){
                        return("<table>"+table+"</table>");
                    }else{
                        return("");
                    }
                }
        
                function pa_pagedetails(data, searchvalue) {
                    var newdata = [];
                    for ( var i = 0; i < data["rows"].length; i = i + 1 ){
                        var sant=1;
                        for ( var j = 0; j < newdata.length; j = j + 1 ){
                            if (data["rows"][i][0]+data["rows"][i][1]+data["rows"][i][2]+data["rows"][i][3]==newdata[j][0]+newdata[j][1]+newdata[j][2]+newdata[j][3]){
                                newdata[j][6] = parseInt(newdata[j][6]) + parseInt(data["rows"][i][6]);
                                sant = 0;
                            }
                        }
                        if (sant){
                            newdata.push(data["rows"][i].slice());
                        }
                    }
        
                    var countrfr = 0;
                    var countkwd = 0;
                    var countdrt = 0;
                    var countscl = 0;
                    var tablerfr = "";
                    var tablekwd = "";
                    var tablescl = "";
                    var tabledrt = "";
                    for ( var i = 0; i < newdata.length; i = i + 1 ) {
                        if (newdata[i][0] == searchvalue){
                            var pagetitle = newdata[i][5];
                            switch (newdata[i][3]){
                                case "REFERRAL":    countrfr += parseInt(newdata[ i ][6]);
                                                    tablerfr += "<tr><td class=\'pa-pgdetailsl\'>"+newdata[i][1]+"</td><td class=\'pa-pgdetailsr\'>"+newdata[ i ][6]+"</td></tr>";
                                                    break;
                                case "ORGANIC":     countkwd += parseInt(newdata[ i ][6]);
                                                    tablekwd += "<tr><td class=\'pa-pgdetailsl\'>"+newdata[i][2]+"</td><td class=\'pa-pgdetailsr\'>"+newdata[ i ][6]+"</td></tr>";
                                                    break;
                                case "SOCIAL":      countscl += parseInt(newdata[ i ][6]);
                                                    tablescl += "<tr><td class=\'pa-pgdetailsl\'>"+newdata[i][1]+"</td><td class=\'pa-pgdetailsr\'>"+newdata[ i ][6]+"</td></tr>";
                                                    break;
                                case "DIRECT":      countdrt += parseInt(newdata[ i ][6]);
                                                    break;
                            };
                        };
                    };
                    if (countrfr){
                        tablerfr = "' . __ ( "REFERRALS", 'wp-analytify' ) . ' ("+countrfr+")"+tablerfr+"";
                    }
                    if (countkwd){
                        tablekwd = "' . __ ( "KEYWORDS", 'wp-analytify' ) . ' ("+countkwd+")"+tablekwd+"";
                    }
                    if (countscl){
                        tablescl = "' . __ ( "SOCIAL", 'wp-analytify' ) . ' ("+countscl+")"+tablescl+"";
                    }
                    if (countdrt){
                        tabledrt = "' . __ ( "DIRECT", 'wp-analytify' ) . ' ("+countdrt+")";
                    }
                    return (pagetitle);
                 }
            
                 function online_refresh(){
                    jQuery.post(ajaxurl, {action: "pa_get_online_data", pa_security: "'.wp_create_nonce('pa_get_online_data').'"}, function(response){
                        var data = jQuery.parseJSON(response);
                        if (data["totalsForAllResults"]["ga:activeVisitors"]!==document.getElementById("pa-online").innerHTML){
                            jQuery("#pa-online").fadeOut("slow");
                            jQuery("#pa-online").fadeOut(500);
                            jQuery("#pa-online").fadeOut("slow", function() {
                                if ((parseInt(data["totalsForAllResults"]["ga:activeVisitors"]))<(parseInt(document.getElementById("pa-online").innerHTML))){
                                }else{
                                }
                                document.getElementById("pa-online").innerHTML = data["totalsForAllResults"]["ga:activeVisitors"];
                            });
                            jQuery("#pa-online").fadeIn("slow");
                            jQuery("#pa-online").fadeIn(500);
                            jQuery("#pa-online").fadeIn("slow", function() {
                                
                            });
                        };
        
                        var pagepath = [];
                        var referrals = [];
                        var keywords = [];
                        var social = [];
                        var visittype = [];
                        if(data["rows"]){
                        for ( var i = 0; i < data["rows"].length; i = i + 1 ) {
                            pagepath.push( data["rows"][ i ][0] );
                            if (data["rows"][i][3]=="REFERRAL"){
                                referrals.push( data["rows"][ i ][1] );
                            }
                            if (data["rows"][i][3]=="ORGANIC"){
                                keywords.push( data["rows"][ i ][2] );
                            }
                            if (data["rows"][i][3]=="SOCIAL"){
                                social.push( data["rows"][ i ][1] );
                            }
                            visittype.push( data["rows"][ i ][3] );
                        }
                    }
                        var upagepath = pagepath.filter(onlyUniqueValues);
                        var upagepathstats = [];
                        for ( var i = 0; i < upagepath.length; i = i + 1 ) {
                            upagepathstats[i]={"pagepath":upagepath[i],"count":countvisits(data,upagepath[i])};
                        }
                        upagepathstats.sort( function(a,b){ return b.count - a.count } );
        
                        var pgstatstable = "";
                        for ( var i = 0; i < upagepathstats.length; i = i + 1 ) {
                            if (i < 10 ){
                                pgstatstable += "<tr class=\"pa-pline\"><td class=\"pa-pright\">"+(i+1)+"</td><td class=\"pa-pleft\"><a href=\"'.get_option("pt_webprofile_url").'"+upagepathstats[i].pagepath.substring(0,70)+"\" title=\""+pa_pagedetails(data, upagepathstats[i].pagepath)+"\" target=\"_blank\">"+pa_pagedetails(data, upagepathstats[i].pagepath)+"</a></td><td class=\"pa-pright\">"+upagepathstats[i].count+"</td></tr>";
                            }
                        }
                        document.getElementById("pa-pages").innerHTML="<br /><table class=\"pa-pg Real_Time_Statistics_table\"><tr><th class=\"wd_1\">#</th><th>Page Title</th><th class=\"wd_2\">Visitors</th></tr>"+pgstatstable+"</table>";
        
                        var ureferralsstats = [];
                        var ureferrals = referrals.filter(onlyUniqueValues);
                        for ( var i = 0; i < ureferrals.length; i = i + 1 ) {
                            ureferralsstats[i]={"value":ureferrals[i],"count":countvisits(data,ureferrals[i])};
                        }
                        ureferralsstats.sort( function(a,b){ return b.count - a.count } );
        
                        var ukeywordsstats = [];
                        var ukeywords = keywords.filter(onlyUniqueValues);
                        for ( var i = 0; i < ukeywords.length; i = i + 1 ) {
                            ukeywordsstats[i]={"value":ukeywords[i],"count":countvisits(data,ukeywords[i])};
                        }
                        ukeywordsstats.sort( function(a,b){ return b.count - a.count } );
        
                        var usocialstats = [];
                        var usocial = social.filter(onlyUniqueValues);
                        for ( var i = 0; i < usocial.length; i = i + 1 ) {
                            usocialstats[i]={"value":usocial[i],"count":countvisits(data,usocial[i])};
                        }
                        usocialstats.sort( function(a,b){ return b.count - a.count } );
        
                        var uvisittype = ["REFERRAL","ORGANIC","SOCIAL"];
                        document.getElementById("pa-tdo-right").innerHTML = "<div class=\"pa-bigtext\"><span class=\"count-visits\">" +countvisits(data,uvisittype[0])+ "</span><span class=\"source\"><a href=\"#\" title=\""+pa_generatetooltip(ureferralsstats)+"\">"+\'' . __ ( "REFERRAL", 'wp-analytify' ) . '\'+"</a></span></div>";
                        document.getElementById("pa-tdo-right").innerHTML += "<div class=\"pa-bigtext\"><span class=\"count-visits\">" +countvisits(data,uvisittype[1])+ "</span><span class=\"source\"><a href=\"#\" title=\""+pa_generatetooltip(ukeywordsstats)+"\">"+\'' . __ ( "ORGANIC", 'wp-analytify' ) . '\'+"</a></span></div>";
                        document.getElementById("pa-tdo-right").innerHTML += "<div class=\"pa-bigtext\"><span class=\"count-visits\">" +countvisits(data,uvisittype[2])+ "</span><span class=\"source\"><a href=\"#\" title=\""+pa_generatetooltip(usocialstats)+"\">"+\'' . __ ( "SOCIAL", 'wp-analytify' ) . '\'+"</a></span></div>";
        
                        var uvisitortype = ["DIRECT","NEW","RETURNING"];
                        document.getElementById("pa-tdo-rights").innerHTML = "<div class=\"pa-bigtext\"><span class=\"count-visits\">" +countvisits(data,uvisitortype[0])+ "</span><span class=\"source\"><a href=\"#\" title=\"DIRECT Visitors\">"+\'' . __ ( "DIRECT", 'wp-analytify' ) . '\'+"</a></span></div>";
                        document.getElementById("pa-tdo-rights").innerHTML += "<div class=\"pa-bigtext\"><span class=\"count-visits\">" +countvisits(data,uvisitortype[1])+ "</span><span class=\"source\"><a href=\"#\" title=\"NEW Visitors\">"+\'' . __ ( "NEW", 'wp-analytify' ) . '\'+"</a></span></div>";
                        document.getElementById("pa-tdo-rights").innerHTML += "<div class=\"pa-bigtext\"><span class=\"count-visits\">" +countvisits(data,uvisitortype[2])+ "</span><span class=\"source\"><a href=\"#\" title=\"RETURNING Visitors\">"+\'' . __ ( "RETURNING", 'wp-analytify' ) . '\'+"</a></span></div>";
        
                        if (!data["totalsForAllResults"]["ga:activeVisitors"]){
                            location.reload();
                        }
        
                    });
               };
               online_refresh();
               setInterval(online_refresh, 20000);
               </script>';
            return $code;
    }

    public function pa_realtime_data() {
            $profile_id = get_option("pt_webprofile_dashboard");
            $metrics = 'ga:activeVisitors';
            $dimensions = 'ga:pagePath,ga:source,ga:keyword,ga:trafficType,ga:visitorType,ga:pageTitle';
            try {
               
                    $data = $this->service->data_realtime->get ( 'ga:' . $profile_id, $metrics, array (
                            'dimensions' => $dimensions
                    ) );
                }
             catch ( Exception $e ) {
                update_option ( 'pa_lasterror_occur', esc_html($e));
                return '';
            }
            return $data;           
    }

    public function pa_welcome_message() {
            $pointer_content = '<h3>Analytify - Google Analytics for WordPress.</h3>';
            $pointer_content .= '<p>Thank you for activating Analytify Plugin. Enjoy Google Analytics for everything in WordPress.</p>';
                ?>
               <script type="text/javascript">
               //<![CDATA[
               jQuery(document).ready( function($) {
                $('#toplevel_page_pa-dashboard').pointer({
                    content: '<?php echo $pointer_content; ?>',
                    position: 'left',
                    close: function() {
                        <?php update_option("pa_welcome_message",1) ?>
                    }
                  }).pointer('open');
               });
               //]]>
               </script>
        <?php
    }

    function analytify_plugin_updater() {

        // retrieve our license key from the DB
        $wpa_license_key = trim( get_option( 'analytify_license_key' ) );
        //$wpa_license_key = 'f77150f419e3128874f401cf832552bc';
        // setup the updater
        $edd_updater = new EDD_SL_Plugin_Updater( ANALYTIFY_STORE_URL, __FILE__, array( 
                'version'   => '1.0.5',               // current version number
                'license'   => $wpa_license_key,        // license key (used get_option above to retrieve from DB)
                'item_name' => ANALYTIFY_PRODUCT_NAME,    // name of this plugin
                'author'    => 'Muhammad Adnan'  // author of this plugin
            )
        );

    }


    function wp_analytify_register_option() {
        // creates our settings in the options table
        register_setting('analytify-settings', 'analytify_license_key', array($this, 'analytify_sanitize_license' ));
    }


    function analytify_sanitize_license( $new ) {
      $old = get_option( 'analytify_license_key' );
      if( $old && $old != $new ) {
        delete_option( 'analytify_license_status' ); // new license has been entered, so must reactivate
      }
      return $new;
    }

    /************************************
    * Activate a license key
    *************************************/

    function analytify_activate_license() {

      // listen for our activate button to be clicked
      if( isset( $_POST['analytify_license_activate'] ) ) {
        
        // run a quick security check 
        if( ! check_admin_referer( 'analytify_nonce', 'analytify_nonce' ) )   
          return; // get out if we didn't click the Activate button

        // retrieve the license from the database
        $license = trim( get_option( 'analytify_license_key' ) );
          

        // data to send in our API request
        $api_params = array( 
          'edd_action'=> 'activate_license',
          'license'   => $license, 
          'item_name' => urlencode( ANALYTIFY_PRODUCT_NAME ), // the name of our product in EDD
          'url'       => home_url()
        );
        
        // Call the custom API.
        $response = wp_remote_get( add_query_arg( $api_params, ANALYTIFY_STORE_URL ), array( 'timeout' => 15, 'sslverify' => false ) );

        // make sure the response came back okay
        if ( is_wp_error( $response ) )
          return false;

        // decode the license data
        $license_data = json_decode( wp_remote_retrieve_body( $response ) );
      

        // $license_data->license will be either "valid" or "invalid"

        update_option( 'analytify_license_status', $license_data->license );

      }
    }

    /***********************************************
    * Illustrates how to deactivate a license key.
    * This will descrease the site count
    ***********************************************/

    function analytify_deactivate_license() {

      // listen for our activate button to be clicked
      if( isset( $_POST['analytify_license_deactivate'] ) ) {

        // run a quick security check 
        if( ! check_admin_referer( 'analytify_nonce', 'analytify_nonce' ) )   
          return; // get out if we didn't click the Activate button

        // retrieve the license from the database
        $license = trim( get_option( 'analytify_license_key' ) );
          

        // data to send in our API request
        $api_params = array( 
          'edd_action'=> 'deactivate_license', 
          'license'   => $license, 
          'item_name' => urlencode( ANALYTIFY_PRODUCT_NAME ), // the name of our product in EDD
          'url'       => home_url()
        );
        
        // Call the custom API.
        $response = wp_remote_get( add_query_arg( $api_params, ANALYTIFY_STORE_URL ), array( 'timeout' => 15, 'sslverify' => false ) );

        // make sure the response came back okay
        if ( is_wp_error( $response ) )
          return false;

        // decode the license data
        $license_data = json_decode( wp_remote_retrieve_body( $response ) );
        
        // $license_data->license will be either "deactivated" or "failed"
        if( $license_data->license == 'deactivated' )
          delete_option( 'analytify_license_status' );
      }
    }

    
    function analytify_check_license() {

      global $wp_version;

      $license = trim( get_option( 'analytify_license_key' ) );

      $api_params = array( 
        'edd_action' => 'check_license', 
        'license' => $license, 
        'item_name' => urlencode( ANALYTIFY_PRODUCT_NAME ),
        'url'       => home_url()
      );

      // Call the custom API.
      $response = wp_remote_get( add_query_arg( $api_params, ANALYTIFY_STORE_URL ), array( 'timeout' => 15, 'sslverify' => false ) );

      if ( is_wp_error( $response ) )
        return false;

      $license_data = json_decode( wp_remote_retrieve_body( $response ) );

      if( $license_data->license == 'valid' ) {
        return 'valid';
        // this license is still valid
      } else {

        return 'invalid';
        // this license is no longer valid
      }
    }


}

$wp_analytify =   new WP_Analytify();
$wp_analytify->pa_check_warnings();

} //end if
?>